@@ -4,11 +4,12 @@ import operator
 import warnings
 from typing import Any, Callable, List, Optional, Tuple, Union
 
-import e3nn_jax as e3nn
 import jax
 import jax.numpy as jnp
 import jax.scipy
 import numpy as np
+
+import e3nn_jax as e3nn
 from e3nn_jax import Irreps, axis_angle_to_angles, config, matrix_to_angles, quaternion_to_angles
 from e3nn_jax._src.irreps import IntoIrreps
 
@@ -403,7 +404,7 @@ class IrrepsArray:
                     mul, ir = self.irreps[i - 1]
                     if (start - self.irreps[: i - 1].dim) % ir.dim == 0:
                         mul1 = (start - self.irreps[: i - 1].dim) // ir.dim
-                        return self.convert(
+                        return self._convert(
                             self.irreps[: i - 1] + e3nn.Irreps([(mul1, ir), (mul - mul1, ir)]) + self.irreps[i:]
                         )[index]
 
@@ -416,7 +417,7 @@ class IrrepsArray:
                     mul, ir = self.irreps[i - 1]
                     if (stop - self.irreps[: i - 1].dim) % ir.dim == 0:
                         mul1 = (stop - self.irreps[: i - 1].dim) // ir.dim
-                        return self.convert(
+                        return self._convert(
                             self.irreps[: i - 1] + e3nn.Irreps([(mul1, ir), (mul - mul1, ir)]) + self.irreps[i:]
                         )[index]
 
@@ -500,7 +501,7 @@ class IrrepsArray:
             >>> IrrepsArray("0e + 0x1e + 0e", jnp.ones(2)).simplify()
             2x0e [1. 1.]
         """
-        return self.convert(self.irreps.simplify())
+        return self._convert(self.irreps.simplify())
 
     def unify(self) -> "IrrepsArray":
         r"""Unify the irreps.
@@ -509,9 +510,9 @@ class IrrepsArray:
             >>> IrrepsArray("0e + 0x1e + 0e", jnp.ones(2)).unify()
             1x0e+0x1e+1x0e [1. 1.]
         """
-        return self.convert(self.irreps.unify())
+        return self._convert(self.irreps.unify())
 
-    def sorted(self) -> "IrrepsArray":
+    def sort(self) -> "IrrepsArray":
         r"""Sort the irreps.
 
         Example:
@@ -521,7 +522,20 @@ class IrrepsArray:
         irreps, p, inv = self.irreps.sort()
         return IrrepsArray.from_list(irreps, [self.list[i] for i in inv], self.shape[:-1])
 
-    def filtered(self, keep_ir: Union[e3nn.Irreps, List[e3nn.Irrep], Callable[[e3nn.MulIrrep], bool]]) -> "IrrepsArray":
+    sorted = sort
+
+    def regroup(self) -> "IrrepsArray":
+        r"""Regroup the same irreps together.
+
+        Equivalent to :meth:`sorted` followed by :meth:`simplify`.
+
+        Example:
+            >>> IrrepsArray("0e + 1o + 2x0e", jnp.arange(6)).regroup()
+            3x0e+1x1o [0 4 5 1 2 3]
+        """
+        return self.sorted().simplify()
+
+    def filter(self, keep_ir: Union[e3nn.Irreps, List[e3nn.Irrep], Callable[[e3nn.MulIrrep], bool]]) -> "IrrepsArray":
         r"""Filter the irreps.
 
         Args:
@@ -539,6 +553,8 @@ class IrrepsArray:
             irreps, [x for x, mul_ir in zip(self.list, self.irreps) if mul_ir in irreps], self.shape[:-1]
         )
 
+    filtered = filter
+
     @property
     def slice_by_mul(self):
         r"""Return the slice with respect to the multiplicities.
@@ -566,20 +582,26 @@ class IrrepsArray:
         """
         return _ChunkIndexSliceHelper(self)
 
-    def repeat_irreps_by_last_axis(self) -> "IrrepsArray":
+    def axis_to_irreps(self, axis: int = -2) -> "IrrepsArray":
         r"""Repeat the irreps by the last axis of the array.
 
         Example:
             >>> x = IrrepsArray("0e + 1e", jnp.arange(2 * 4).reshape(2, 4))
-            >>> x.repeat_irreps_by_last_axis()
+            >>> x.axis_to_irreps()
             1x0e+1x1e+1x0e+1x1e [0 1 2 3 4 5 6 7]
         """
-        assert len(self.shape) >= 2
-        irreps = (self.shape[-2] * self.irreps).simplify()
-        array = self.array.reshape(self.shape[:-2] + (irreps.dim,))
-        return IrrepsArray(irreps, array)
+        assert self.ndim >= 2
+        axis = _standardize_axis(axis, self.ndim)[0]
+        jnp = _infer_backend(self.array)
 
-    def factor_irreps_to_last_axis(self) -> "IrrepsArray":  # noqa: D102
+        new_irreps = (self.shape[axis] * self.irreps).simplify()
+        new_array = jnp.moveaxis(self.array, axis, -2)
+        new_array = jnp.reshape(new_array, self.shape[:-2] + (new_irreps.dim,))
+        return IrrepsArray(new_irreps, new_array)
+
+    repeat_irreps_by_last_axis = axis_to_irreps
+
+    def irreps_to_axis(self) -> "IrrepsArray":  # noqa: D102
         raise NotImplementedError
 
     # Move multiplicity to the previous last axis and back
@@ -714,7 +736,7 @@ class IrrepsArray:
         k = (1 - d) / 2
         return self.transform_by_angles(*matrix_to_angles(R), k)
 
-    def convert(self, irreps: IntoIrreps) -> "IrrepsArray":
+    def _convert(self, irreps: IntoIrreps) -> "IrrepsArray":
         r"""Convert the list property into an equivalent irreps.
 
         Args:
@@ -728,7 +750,7 @@ class IrrepsArray:
 
         Example:
             >>> x = IrrepsArray.from_list("6x0e + 4x0e", [None, jnp.ones((4, 1))], ())
-            >>> x.convert("5x0e + 5x0e").list
+            >>> x._convert("5x0e + 5x0e").list
             [None, DeviceArray([[0.],
                          [1.],
                          [1.],
@@ -1064,7 +1086,7 @@ def norm(array: IrrepsArray, *, squared: bool = False) -> IrrepsArray:
 
 def normal(
     irreps: IntoIrreps,
-    key: jnp.ndarray,
+    key: jnp.ndarray = None,
     leading_shape: Tuple[int, ...] = (),
     *,
     normalize: bool = False,
@@ -1074,7 +1096,7 @@ def normal(
 
     Args:
         irreps (Irreps): irreps of the output array
-        key (jnp.ndarray): random key
+        key (jnp.ndarray): random key (if not provided, use the hash of the irreps as seed, usefull for debugging)
         leading_shape (tuple of int): shape of the leading dimensions
         normalize (bool): if True, normalize the output array
         normalization (str): normalization of the output array, ``"component"`` or ``"norm"``
@@ -1086,6 +1108,8 @@ def normal(
 
     Examples:
         >>> np.set_printoptions(precision=2, suppress=True)
+        >>> e3nn.normal("1o").shape
+        (3,)
 
         Generate a random array with normalization ``"component"``
 
@@ -1116,6 +1140,10 @@ def normal(
     if normalization is None:
         normalization = config("irrep_normalization")
 
+    if key is None:
+        warnings.warn("e3nn.normal: the key (random seed) is not provided, use the hash of the irreps as key!")
+        key = jax.random.PRNGKey(hash(irreps))
+
     if normalize:
         list = []
         for mul, ir in irreps:
@@ -1191,7 +1219,7 @@ class _IndexUpdateRef:
             if self.irreps.simplify() != values.irreps.simplify():
                 raise ValueError("The irreps of the array and the values to set must be the same.")
 
-            values = values.convert(self.irreps)
+            values = values._convert(self.irreps)
 
             def fn(x, y, mul, ir):
                 if x is not None and y is not None:
@@ -1242,7 +1270,7 @@ class _IndexUpdateRef:
             if self.irreps.simplify() != values.irreps.simplify():
                 raise ValueError("The irreps of the array and the values to add must be the same.")
 
-            values = values.convert(self.irreps)
+            values = values._convert(self.irreps)
 
             def fn(x, y, mul, ir):
                 if x is not None and y is not None:
